(import
  [collections [OrderedDict]]
  [numpy :as np]
  [pandas :as pd]
  [sklearn.linear-model :as sklm]
  [kodhy.util [rmse kfold-cv-pred with-1o-interacts]])

(defn ridge-f [x-train y-train &optional x-test x-colnames]
  (setv m (kwc sklm.RidgeCV
    :scoring "neg_mean_squared_error"
    :alphas (** 10. (np.arange -5 6))))
  (.fit m x-train y-train)
  (if (none? x-colnames)
    (.predict m x-test)
    (kwc pd.Series :index (+ ["Intercept"] (list x-colnames))
      (np.append [m.intercept_] m.coef_))))

(setv models [
  ["trivial" :no-interactions (fn [x-train y-train &optional x-test x-colnames]
    (when (none? x-colnames)
      (np.repeat y-train (len x-test))))]
  ["ridge" :no-interactions ridge-f]
  ["ridge-interact" :interactions ridge-f]])

(defn pred-outcome [dv timepoint obs]
  (setv obsT (ss obs (<= $timepoint timepoint)))
  (setv d (cbind
    :mentor_mother (.astype ($ (= (.set-index (.drop-duplicates (getl obsT : (qw s condition))) "s")
      "Mentor_Mother") condition) float)
    (.applymap
      (cbind
        (kwc .rename (.pivot obsT "s" "timepoint" "alcohol_any")
          :columns (λ (+ "alcohol_any_" it)))
        ($ (.set-index (ss obs (= $timepoint "5_years")) "s") alcohol_any_while_preg))
      (λ (get {"Sober" 0 "Drinking" 1 np.nan np.nan} it)))
    :y (getl (.pivot obsT "s" "timepoint" dv) : timepoint)))
  (assert (= (len d) (len (.unique ($ obsT s)))))
  (setv d (ss d (pd.notnull $y)))
  ; Fill missing values with column modes.
  (setv d (.fillna d (geti (.mode d) 0)))

  (setv y (. ($ d y) values))
  (setv X (kwc .drop :axis 1 d "y"))
  (setv Xinter (cbind
    (kwc .drop :axis 1 d "y")
    (kwc .rename
      (.apply (getl d : (filt (.startswith it "alcohol_any_") d.columns))
        (λ (* ($ d mentor_mother) it)))
      :columns (λ (+ "mm_" it)))))

  (setv results (OrderedDict (rmap [[model-name x-kind model-f] models]
    (setv my-X (if (= x-kind :interactions) Xinter X))
    (setv y-pred (kwc kfold-cv-pred (.as-matrix my-X) y model-f
      :n-folds 10 :+shuffle :random-state 15))
    (, model-name {
      "rmse" (rmse y-pred y)
      "on_all_data" (kwc model-f (.as-matrix my-X) y :x-colnames my-X.columns)})))))

(defn rmse-table [pred-outcome-results]
  (pd.concat (rmap [[k d] (.items pred-outcome-results)] (kwc pd.DataFrame
    :index [k]
    :columns ["RMSE"]
    [(get d "rmse")]))))

;  (rmap [[_ g] (.groupby obs "s")]
;    (setv y (get (geti g (dec (len g))) dv))
;    (setv mentor-mother (= (get (geti g 0) "condition") "Mentor_Mother"))



